function MDMO_feature = MDMO(casmeID, useOriginalFps, useAffine)
%This function calculates the MDMO_feature of a given database
%
% input:
% --casmeID:          1 or 2. 1 for 'CASME', 2 for 'CASMEII';
% --useOriginalFps:   0 or 1. 
%                     0: use normalized number of frames
%                     1: use original fps
% --useAffine:        0 or 1. 0 for 'No', 1 for 'Yes'.
%
% output:  MDMO feature.

if ~exist('casmeID', 'var')
    casmeID = 1;
end

if ~exist('useOriginalFps', 'var')
    useOriginalFps = true;
end

if ~exist('useAffine', 'var')
    useAffine = true;
end

if casmeID < 1 || casmeID > 2
    casmeID = 1;
end

if casmeID == 1
    load('casme_stru.mat');
    videoLength = 70;       %normalized number of frames
    rootPath = 'E:\DATA\micro expression data\CASME1\selected\'; %file path of database
else
    %load('..\DataStru\casme2_stru.mat');
    load('casme2_stru.mat');
    videoLength = 150;      %normalized number of frames
    rootPath = 'E:\DATA\micro expression data\CASME2\CASME2-RAW\'; %file path of database
end

% initilization
nSamples = size(data_stru,2);
MDMO_feature = zeros(nSamples, 36 * 3);

% get MDMO feature
for i = 1 : nSamples
    disp(['Processing video: ' num2str(i) '/' num2str(nSamples)]);
    feature_points = int32(data_stru(i).imgfile(1).feature_points);
    
    %include image list
    imgPath = [rootPath 'sub' num2str(data_stru(i).subject,'%02d') '\' data_stru(i).material '\' data_stru(i).imgfile(1).name]; 
    img = rgb2gray(imread(imgPath));
    [height, width] = size(img);
    
    video = zeros(height, width, data_stru(i).nFrames);
    for j = 1 : data_stru(i).nFrames
        imgPath = [rootPath 'sub' num2str(data_stru(i).subject,'%02d') '\' data_stru(i).material '\' data_stru(i).imgfile(j).name]; 
        
        img = rgb2gray(imread(imgPath));
        video(:,:,j) = img;
    end
    
    %interpVideo
    if useOriginalFps 
        VolData = video;
    else
        VolData = interpVideoGray(video,videoLength);
    end
    
    %reset the video length
    videoLength = size(VolData, 3);
    
    %get MDMO feature
    [feature_rho, feature_theta] = MDMO_video(VolData, feature_points, useAffine);
    MDMO_feature(i, 1:36) = feature_rho;
    MDMO_feature(i, 37:72) = cos(feature_theta);
    MDMO_feature(i, 73:108) = sin(feature_theta);
end
end

function [feature_rho, feature_theta] = MDMO_video(video, feature_points, useAffine)
    [height, width, nFrames] = size(video);
    
    % ROI
    au_region = loadROI36(feature_points);
    nRegion=size(au_region,2);
    img = video(:,:,1);
    template=img;
    for r=1:nRegion
        au_region(r).Mask = roipoly(template, au_region(r).Points(1,:)',au_region(r).Points(2,:)');
    end
    U = zeros(height, width, nFrames-1);
    V = zeros(height, width, nFrames-1);
    
    for f=2:nFrames
        % calculate the optical flow
        img_f = video(:,:,f);
        [u, v] = RLOF_fun(uint8(img), uint8(img_f), 'RLOF');
        U(:,:,f-1) = u;
        V(:,:,f-1) = v;
        fprintf('%d/%d.\n',f,nFrames);
    end
    
    feature=zeros(2,nRegion,nFrames-1);
    for f = 1: nFrames-1
        if useAffine == 1
            % affine the optical flow
            [u, v] = affine(U(:,:,f), V(:,:,f), feature_points);
        else
            u = U(:,:,f);
            v = V(:,:,f);
        end
        [theta, rho] = cart2pol(u,v);
        % choose the main direction of optical flow
        for r=1:nRegion
            mask=au_region(r).Mask;
            theta1=theta(mask);
            rho1=rho(mask);
            binranges = [-pi, -pi+pi/8, -pi+pi/8*3, -pi+pi/8*5, -pi+pi/8*7, -pi+pi/8*9, -pi+pi/8*11, -pi+pi/8*13, -pi+pi/8*15, pi];
            [bincounts,ind] = histc(theta1,binranges);
            tmp = bincounts;
            tmp(1,1)=tmp(1,1)+tmp(end,1)+tmp(end-1,1);
            bincounts=tmp;
            tmp = tmp(1:end-2,1);
            ind(ind==9)=1;
            ind(ind==10)=1;
            [max_number, max_index] = max(bincounts);
            theta1 = theta1(ind==max_index);
            rho1 = rho1(ind==max_index);
            theta1 = mean(theta1);
            rho1 = mean(rho1);
            feature(1,r,f)=rho1;
            feature(2,r,f)=theta1;
        end  
    end
    
    % weight
    weight_feature=zeros(2,nRegion);
    sum_rho = sum(sum(feature(1,:,:)));
    
    rho_t           = zeros(size(feature, 2), size(feature, 3));            
    theta_t         = zeros(size(feature, 2), size(feature, 3));            
    rho_t(:,:)      = feature(1,:,:);                                       
    theta_t(:,:)    = feature(2,:,:);                                       
    rho_t           = rho_t';                                              
    theta_t         = theta_t';                                             
    [uf, vf]        = pol2cart(theta_t, rho_t);                             
    wuf             = zeros(1,nRegion);                                     
    wvf             = zeros(1,nRegion);                                     
    
    for f = 1 : nFrames-1
        
        alpha = sum(feature(1,:,f))/sum_rho;
        wuf   = wuf + alpha * uf(f,:);                                                                   
        wvf   = wvf + alpha * vf(f,:);                                      
    end

    [feature_theta, feature_rho] = cart2pol(wuf, wvf);                      
    tmp = max(feature_rho);                                                  
    feature_rho = feature_rho /tmp;                                          
end