/******************************************************************************
 * COPYRIGHT:                                                                 * 
 *                                                                            *
 * This file is the property of the author and Communication Systems Group,   *
 * Technische Universitt Berlin. All rights reserved.                        *
 *                                                                            *
 * It may not be publicly disclosed, distributed, used, copied or modified    *
 * without prior written authorization by a representative of                 *
 * Communication Systems Group, Technische Universitt Berlin or the author.  *
 * Any modified version of this document needs to contain this header.        *
 ******************************************************************************/

/******************************************************************************
 * THERMS IF USAGE:                                                           *
 * PERSONAL, NON-COMMERCIAL or ACADEMIC USAGE:                                *
 *   You are free to use this software for whatever you like. If you use this *
 *   algorithm for a scientific publication, please cite the one of the       *
 *   following paper:                                                         *
 *																			  *
 * @INPROCEEDINGS{ICIPSenst2014,											  *
 * AUTHOR = {Tobias Senst and Thilo Borgmann and Ivo Keller and Thomas		  *
 * Sikora},																	  *
 * TITLE = {Cross based Robust Local Optical Flow},							  *
 * BOOKTITLE = {21th IEEE International Conference on Image Processing},	  *
 * YEAR = {2014},															  *
 * MONTH = okt,																  *
 * PAGES = {1967--1971},													  *
 * ADDRESS = {Paris, France},												  *
 * }																		  *
 *																			  *
 * @INPROCEEDINGS{ICIPSenst2013,											  *
 * AUTHOR = {Tobias Senst and Jonas Geistert and Ivo Keller and Thomas		  *
 * Sikora},																	  *
 * TITLE = {Robust Local Optical Flow Estimation using Bilinear Equations for *
 * Sparse Motion Estimation},												  *
 * BOOKTITLE = {20th IEEE International Conference on Image Processing},	  *
 * YEAR = {2013},															  *
 * MONTH = sep,																  *
 * PAGES = {2499--2503},													  *
 * ADDRESS = {Melbourne, Australia},										  *
 * DOI = {10.1109/ICIP.2013.6738515},										  *
 * }																		  *
 *                                                                            *
 * @ARTICLE{TCSVTSenst2012,                                                   *
 * AUTHOR = {Tobias Senst and Volker Eiselein and Thomas Sikora},             *
 * TITLE = {Robust Local Optical Flow for Feature Tracking},                  *
 * JOURNAL = {IEEE Transactions on Circuits and Systems for Video Technology},*
 * YEAR = {2012},                                                             *
 * MONTH = sep,                                                               *
 * PAGES = {1377--1387},                                                      *
 * VOLUME = {22},                                                             *
 * NUMBER = {9},                                                              *
 * DOI = {10.1109/TCSVT.2012.2202070}                                         *
 * }                                                                          *
 *                                                                            *
 * @INPROCEEDINGS{WACVSenst2011,                                              *
 * AUTHOR = {Tobias Senst and Volker Eiselein and Rubn Heras Evangelio and   *
 * Thomas Sikora},                                                            *
 * TITLE = {Robust Modified L2 Local Optical Flow Estimation and Feature      *
 * Tracking},                                                                 *
 * BOOKTITLE = {IEEE Workshop on Motion and Video Computing},                 *
 * YEAR = {2011},                                                             *
 * MONTH = jan,                                                               *
 * EDITOR = {Eric Mortensen},                                                 *
 * PAGES = {685--690},                                                        *
 * ADDRESS = {Kona, USA},                                                     *
 * DOI = {10.1109/WACV.2011.5711571},                                         *
 * }                                                                          *
 *                                                                            *
 * COMMERCIAL USAGE:                                                          *
 *   It is not allowed to use any content of this package for any commercial  *
 *   use or any advertisement for upcoming commercial products. If you want to*
 *   use any content for such a purpose please contact:                       *
 *   Prof. Dr.-Ing. Thomas Sikora <sikora@nue.tu-berlin.de>.                  *
 ******************************************************************************/

/******************************************************************************
 * WARRANTIES:                                                                *
 *                                                                            *
 * Software provided by Technische Universitt Berlin with this document is   *
 * provided "AS IS" and any express of implied warranties including, but      *
 * not limited to, the implied warranties of merchantability and fitness      *
 * for a particular purpose are disclaimed.                                   *
 * In no event shall the author or contributors be liable for any direct,     *
 * indirect, incidental, special, exemplary, or consequential damages         *
 * (including, but not limited to, procurement of substitute goods or         *
 * services, loss of use, data, or profits or business interruption) caused in*
 * any way out of the use of this software, even if advised of the possibility*
 * of such damage.                                                            *
 ******************************************************************************/
/** 
 * \file IFeatureTracker.h
 * \brief RLOF interface.
 * \version 1.2
 * \date 04.05.2012
 * \author Tobias Senst
 */

#pragma once
#ifndef _IFEATURETRACKER_H_
#define _IFEATURETRACKER_H_

#include <OpticalFlow/cvwrapper.hpp>
#ifndef _NOT_USE_OPENCV_
#include <opencv2/core/core.hpp>
#include <opencv2\imgproc\imgproc.hpp>
#else
#include <stdlib.h>
#include <vector>
#define uchar unsigned char 
#endif
#ifdef DECL_STATIC
	#define DECL_SPEC 
#else
#ifndef DECL_SPEC
	#ifdef _USRDLL
		#ifdef WIN32
			 #ifdef _USRDLL
				 #define DECL_SPEC __declspec(dllexport)
			 #else
				 #define DECL_SPEC __declspec(dllimport)
			 #endif // DLL_EXPORT
		#endif // WIN32
		#ifdef __GNUG__
			#define DECL_SPEC __attribute__ ((visibility("default")))
		#endif // __GNUG__
	#endif
    #ifndef DECL_SPEC
        #define DECL_SPEC
    #endif
#endif // DECLSPEC_DEFINED
#endif

/*! \class FeatureTrackerContext
 *  \brief Feature tracker parameter base class
 * This class is the base class for all parameter classes in that library.
 *  
*/
class FeatureTrackerContext
{
public:
	virtual ~FeatureTrackerContext() {};
};

/*! \class FeatureTracker
 *  \brief Feature tracker base class
 * Interface for all PLK/RLOF derivatives in the RLOF library.
 *  
*/
class DECL_SPEC IFeatureTracker
{
public:
	virtual ~IFeatureTracker() {};
	/*! Track a set of features points (prevPoints) that are specified by a 2D point list and 
	 * store the tracked positions at the (currPoints) point list by using the image informations
	 * providen by the prevImage and currImage. If these images has to be gray scales images. Parameters are set by the context argument.
	 * This interface is used to ensure compatibility down to OpenCV 2.0 und upon new OpenCV versions. To ensure such compatibility 
     * OpenCV defined classes are replaced by wrapper classes, see CRPoint.(If use used OpenCV 2.4.6 
	 * the second interface function could be used) 
	*\param imgPtr1 Data pointer to an rgb or gray scaled image at time t. 
	*\param step1 Specifies the number of elements used to allocate a row of image 1.
	*\param imgPtr2 Data pointer to an rgb or gray scaled image at time t+1.
	*\param step2 Specifies the number of elements used to allocate a row of image 2.
	*\param nRows Number of rows of the input images.
	*\param nCols Number of columns of the input images.
	*\param prevPoints Features as point list at time t.
	*\param currPoints Tracked features as point list to time t+1.
	*\param context Context including parameters, if NULL default parameters will be used.
	*/
	virtual void run(
		const uchar * imgPtr1, 
		unsigned int step1,
		const uchar * imgPtr2,
		unsigned int step2,
		unsigned int nRows,
		unsigned int nCols,
		std::vector<CRPoint> & prevPoints,
		std::vector<CRPoint> & currPoints, 
		const FeatureTrackerContext * context){};

	/*! ATTENTION - should be only just with OpenCV 2.4.6. Track a set of features points (prevPoints) that are specified by a 2D point list and 
	 * store the tracked positions at the (currPoints) point list by using the image informations
	 * provided by the prevImage and currImage. If these images are RGB images, they will be transformed
	 * to grav scaled images internally. Parameters are set by the context argument.
	*\param prevImage Image at time t (CV_8UC3 or CV_8UC1 depending from method).
	*\param currImage Image at time t+1 (CV_8UC3 or CV_8UC1 depending from method).
	*\param prevPoints Features as point list at time t.
	*\param currPoints Tracked features as point list to time t+1.
	*\param context Context including parameters, if NULL default parameters will be used.
	*/
	#ifndef _NOT_USE_OPENCV_
	virtual void run(
		const cv::Mat & prevImage, 
		const cv::Mat & currImage, 
		std::vector<cv::Point2f> & prevPoints,
		std::vector<cv::Point2f> & currPoints, 
		const FeatureTrackerContext * context) = 0;
	#endif
	/*! Create a default context containing the default parameter for the related tracking methods.
	*\return context containing default parameter
	*/
	virtual FeatureTrackerContext * createDefaultParameter() = 0;
};

/*! \relates RLOFFactory
 \interface IFeatureTracker "include\OpticalFlow\IFeatureTracker.h"
 \brief RLOF Interface
 
 This class provides a general interface for feature tracking purposes. 
 The RLOF method is a inherent class that is instantiated by the RLOFFactory.
 To parametrize the RLOF tracking method the FeatureTrackerContext is used. Default
 parameters could be taken from the createDefaultParameter function.
 
*/
#endif