/******************************************************************************
 * COPYRIGHT:                                                                 * 
 *                                                                            *
 * This file is the property of the author and Communication Systems Group,   *
 * Technische Universitt Berlin. All rights reserved.                        *
 *                                                                            *
 * It may not be publicly disclosed, distributed, used, copied or modified    *
 * without prior written authorization by a representative of                 *
 * Communication Systems Group, Technische Universitt Berlin or the author.  *
 * Any modified version of this document needs to contain this header.        *
 ******************************************************************************/

/******************************************************************************
 * THERMS IF USAGE:                                                           *
 * PERSONAL, NON-COMMERCIAL or ACADEMIC USAGE:                                *
 *   You are free to use this software for whatever you like. If you use this *
 *   algorithm for a scientific publication, please cite the one of the       *
 *   following paper:                                                         *
 *																			  *
 * @INPROCEEDINGS{ICIPSenst2014,											  *
 * AUTHOR = {Tobias Senst and Thilo Borgmann and Ivo Keller and Thomas		  *
 * Sikora},																	  *
 * TITLE = {Cross based Robust Local Optical Flow},							  *
 * BOOKTITLE = {21th IEEE International Conference on Image Processing},	  *
 * YEAR = {2014},															  *
 * MONTH = okt,																  *
 * PAGES = {1967--1971},													  *
 * ADDRESS = {Paris, France},												  *
 * }																		  *
 *																			  *
 * @INPROCEEDINGS{ICIPSenst2013,											  *
 * AUTHOR = {Tobias Senst and Jonas Geistert and Ivo Keller and Thomas		  *
 * Sikora},																	  *
 * TITLE = {Robust Local Optical Flow Estimation using Bilinear Equations for *
 * Sparse Motion Estimation},												  *
 * BOOKTITLE = {20th IEEE International Conference on Image Processing},	  *
 * YEAR = {2013},															  *
 * MONTH = sep,																  *
 * PAGES = {2499--2503},													  *
 * ADDRESS = {Melbourne, Australia},										  *
 * DOI = {10.1109/ICIP.2013.6738515},										  *
 * }																		  *
 *                                                                            *
 * @ARTICLE{TCSVTSenst2012,                                                   *
 * AUTHOR = {Tobias Senst and Volker Eiselein and Thomas Sikora},             *
 * TITLE = {Robust Local Optical Flow for Feature Tracking},                  *
 * JOURNAL = {IEEE Transactions on Circuits and Systems for Video Technology},*
 * YEAR = {2012},                                                             *
 * MONTH = sep,                                                               *
 * PAGES = {1377--1387},                                                      *
 * VOLUME = {22},                                                             *
 * NUMBER = {9},                                                              *
 * DOI = {10.1109/TCSVT.2012.2202070}                                         *
 * }                                                                          *
 *                                                                            *
 * @INPROCEEDINGS{WACVSenst2011,                                              *
 * AUTHOR = {Tobias Senst and Volker Eiselein and Rubn Heras Evangelio and   *
 * Thomas Sikora},                                                            *
 * TITLE = {Robust Modified L2 Local Optical Flow Estimation and Feature      *
 * Tracking},                                                                 *
 * BOOKTITLE = {IEEE Workshop on Motion and Video Computing},                 *
 * YEAR = {2011},                                                             *
 * MONTH = jan,                                                               *
 * EDITOR = {Eric Mortensen},                                                 *
 * PAGES = {685--690},                                                        *
 * ADDRESS = {Kona, USA},                                                     *
 * DOI = {10.1109/WACV.2011.5711571},                                         *
 * }                                                                          *
 *                                                                            *
 * COMMERCIAL USAGE:                                                          *
 *   It is not allowed to use any content of this package for any commercial  *
 *   use or any advertisement for upcoming commercial products. If you want to*
 *   use any content for such a purpose please contact:                       *
 *   Prof. Dr.-Ing. Thomas Sikora <sikora@nue.tu-berlin.de>.                  *
 ******************************************************************************/

/******************************************************************************
 * WARRANTIES:                                                                *
 *                                                                            *
 * Software provided by Technische Universitt Berlin with this document is   *
 * provided "AS IS" and any express of implied warranties including, but      *
 * not limited to, the implied warranties of merchantability and fitness      *
 * for a particular purpose are disclaimed.                                   *
 * In no event shall the author or contributors be liable for any direct,     *
 * indirect, incidental, special, exemplary, or consequential damages         *
 * (including, but not limited to, procurement of substitute goods or         *
 * services, loss of use, data, or profits or business interruption) caused in*
 * any way out of the use of this software, even if advised of the possibility*
 * of such damage.                                                            *
 ******************************************************************************/
/** 
 * \file RLOFParameter.h
 * \brief RLOF parameter
 
 The file contains the parameter class used by the RLOF method.
 * \version 1.2
 * \date 17.11.2014
 * \author Tobias Senst
 */


#pragma once 
#ifndef _RLOFPARAMETER_H_
#define _RLOFPARAMETER_H_					/*!< No comment*/
#include <OpticalFlow/IFeatureTracker.h>
#ifdef USE_BOOSTSERIALIZATION
#include <boost/lexical_cast.hpp>
#include <boost/archive/xml_oarchive.hpp>
#include <boost/archive/xml_iarchive.hpp>
#include <boost/serialization/nvp.hpp>
#include <boost/serialization/list.hpp>
#include <boost/serialization/vector.hpp>
#include <boost/serialization/map.hpp>
#include <boost/serialization/deque.hpp>
#include <boost/serialization/string.hpp>
#include <boost/serialization/version.hpp>
#include <boost/serialization/split_member.hpp>
#include <boost/serialization/shared_ptr.hpp>
#include <boost/serialization/split_free.hpp>
#include <boost/mpl/bool.hpp>
#include <boost/cast.hpp>
#include <fstream>
#endif

#define RLOF_RESOLVE_TENSOR_AS_L2NORM  16 /*!< recompute the Tensor without the Gradient */
#define RLOF_RESOLVE_TENSOR_CONTINIOUS 32 /*!< recompute the Tensor with the Gradient */
#define RLOF_RESOLVE_USE_I			   64 /*!< use I for the tensor */

/*! \class CLocalFlowParameter
 *  \brief PLK/RLOF parameter base class
 *  The parameter base class provides basic interfaces to set up the algorithm parameter such as number of levels. 
 *  The parameter class is associated to all methods.
*/
class DECL_SPEC CLocalFlowParameter : public FeatureTrackerContext
{
	public:
	/*! Enumeration specifying the support region construction.
	*/
	enum WindowType { WIN_STANDART = 0,		 /*!< Rectangle support region.*/
					  WIN_CROSSADAPTIVE = 2, /*!< Cross based adaptive support region, described in <a href="index.html#Senst2014">Senst et al. 2014</a> .*/
					  WIN_SLIC = 4 };		 /*!<  not used */
	/*! Enumeration specifying the iterative refinement strategy.
	*/
	enum SolverType { ITER_STANDART = 0, /*!< Standard iterative solver. */
					  ITER_BE		= 2, /*!< Bilinear equation solver, described in <a href="index.html#Senst2013">Senst et al. 2013</a> .*/
					};
	/*! Enumeration will be not used.
	*/
	enum IlluminationModelType { ILL_NO_MODEL = 0,		    /*!<Use standard brightness constancy assumption.*/
								 ILL_GAIN_MODEL = 2,		/*!<not used.*/ 
								 ILL_CONST_MODEL = 4,		/*!<not used.*/ 
								 ILL_TEST_MODEL = 8  };		/*!<not used.*/ 
	CLocalFlowParameter()
		: m_WindowType(WIN_STANDART)
		, m_MaxIter(20)
		, m_MaxLevel(3)
		, m_WinSize(17)
		, m_SolverType(ITER_STANDART)		
		, m_IlluminationModel(ILL_NO_MODEL)
		, m_Verbosity(0)
		, m_Flags(0)
	{};
	/*! Set/Unset the usage of different iterative solver strategies
     *\param val TODO description
	*/
	void solverType(SolverType val)
	{
		m_SolverType = val;
	};
	/*! Set the number of used pyramid levels.
	 *\param val positive number.
	*/
	void level(int val)
	{
		if(val < 0)
			throw(std::exception("CLocalFlowParameter: error maxLevel need to be >= 0\n"));
		m_MaxLevel = val; 
	};
	/*! Set the number of maximal iterations.
	 *\param val positive number.
	*/
	void iteration(int val)
	{
		m_MaxIter = val; 
	};
	/*! set the verbosity level (debug output level) default 0
	 *\param val Verbosity level
	    - 0 no output
	    - >0 method specific output
	 !*/
	void verbosity(int val)
	{
		m_Verbosity = val;
	};
	/*! Interface to retrieve the used support region shape.
	 *  - WIN_STANDART denotes a rectangle support region or
	 *  - WIN_CROSSADAPTIVE denotes the usage of a cross based adaptive support region, see <a href="index.html#Senst2014">Senst et al. 2014</a>. 
	 *\return enumeration WindowType 
	*/
	WindowType windowType() const						{ return m_WindowType; };
	/*! Interface to retrieve the used solver. 
	 *  - ITER_STANDART denotes the standard iterative solver.
	 *  - ITER_BE denotes the bilinear equation solver, see <a href="index.html#Senst2013">Senst et al. 2013</a>.
	 *\return enumeration SolverType
	*/
	CLocalFlowParameter::SolverType solverType() const	{ return m_SolverType; };
	/*! Interface to retrieve the number of Gaussian pyramid levels used. 
	 *\return Number of pyramid levels.
	*/
	const int level() const								{ return m_MaxLevel; };
	/*! Interface to retrieve the number maximal iterations used for the iterative refinement. 
	 *\return Number of maximal iterations.
	*/
	const int iteration() const							{ return m_MaxIter; };
	/*! Interface to retrieve the verbosity level.
	 * The verbosity or debug level of 0 denotes no output to the std::cout
	  *\return the verbosity level (debug level)
	*/
	const int verbosity() const							{ return m_Verbosity;};

protected:
	WindowType	m_WindowType;						/*!< Enumeration specifying support region construction.*/
	SolverType	m_SolverType;						/*!< Enumeration specifying iterative refinement strategy.*/
	int			m_MaxLevel;							/*!< A positive number specifying the number of levels used for the Gaussian pyramid.*/
	int			m_MaxIter;							/*!< A positive number specifying the maximal number of iteration used for the iterative refinement.*/
	int			m_WinSize;							/*!< A positive number specifying the support region size.*/
	int			m_Verbosity;						/*!< A positive number specifying the debug level.*/
	int			m_Flags;							/*!< A positive number specifying additional program parameter.*/
private:
	IlluminationModelType	m_IlluminationModel;
#ifdef USE_BOOSTSERIALIZATION
	private:
	friend class boost::serialization::access;
	template<class Archive>
	void serialize(Archive & ar, const unsigned int version)
	{
		ar & boost::serialization::make_nvp("m_WindowType", m_WindowType);
		ar & boost::serialization::make_nvp("m_SolverType", m_SolverType);
		ar & boost::serialization::make_nvp("m_IlluminationModel", m_IlluminationModel);
		ar & boost::serialization::make_nvp("m_MaxLevel", m_MaxLevel);
		ar & boost::serialization::make_nvp("m_MaxIter", m_MaxIter);
		ar & boost::serialization::make_nvp("m_WinSize", m_WinSize);
		ar & boost::serialization::make_nvp("m_Verbosity", m_Verbosity);
		ar & boost::serialization::make_nvp("m_Flags", m_Flags);
	}
	public:
	virtual std::string uid() const {
		return boost::lexical_cast<std::string>(m_WindowType) 
			 +boost::lexical_cast<std::string>(m_SolverType) 
			 +boost::lexical_cast<std::string>(m_IlluminationModel) 
			 +boost::lexical_cast<std::string>(m_MaxLevel) 
			 +boost::lexical_cast<std::string>(m_MaxIter) 
			 +boost::lexical_cast<std::string>(m_WinSize) 
			 +boost::lexical_cast<std::string>(m_Verbosity) 
			 +boost::lexical_cast<std::string>(m_Flags);
	}
#endif
};
/*! \class CRobustFlowParameter
 *  \brief RLOF parameter class
 *  The Robust Local Optical Flow parameter class contains additional parameters used for the RLOF method and its derivatives, see <a href="index.html#Senst2012">Senst et al. 2012</a>. 
 *  The parameter class is associated to RLOF, BERLOF, CROSS_RLOF, CROSS_BERLOF.
*/
class DECL_SPEC CRobustFlowParameter : public CLocalFlowParameter
{
public:
	/*! Default constructor with the default parameters
	*/
	CRobustFlowParameter();
	/*! Set the upper bound of the support region size.
	 *\param val positive number.
	*/
	void largeWinSize(const int & val);
	/*! Set the lower bound of the support region size.
	 *\param val positive number <= largeWinSize
	*/
	void smallWinSize(const int & val);
	
	/*! Set the shrinked Hample norm parameter.
	 *\param val vector [&sigma;<sub>1</sub> &sigma;<sub>2</sub>] of positive floating point values.
	*/
	void norm(const std::vector<float> val);
	/*! Set the number of iterations with L2 norm. Maximal iterations = MAX(maximal iterations, L2 iterations.
	 *\param val number of iterations with the L2 estimator before using the mod. L2 estimator.
	*/
	void L2iteration(int val);
	/*! Interface to retrieve the upper bound of the support region.
	 *\return Maximal support regions size.
	*/
	const int largeWinSize() const;
	/*! Interface to retrieve the upper lower of the support region.
	 *\return Minimal support regions size.
	*/
	const int smallWinSize() const ;
	/*! Interface to retrieve the shrinked Hampel norm parameter vector.
	 *\return Hamples norm parameter.
	*/
	const std::vector<float> norm() const;
	/*! Interface to retrieve the number of L2 iterations.
	 *\return Number of L2 iterations.
	*/
	const int L2iteration() const;
	/*! Function is used intern to reallocate memory and has no algorithmic meaning,
	 *\return True if region size was changed.
	*/
	const bool	hasChangedWinSize() const;
	/*! Function returns if due to the window setting rgb data is needed and uses. Mostly for internal use.
	 *\return True if rgb input data is needed.
	*/
	bool needRGBData() const;
	
protected:
	// ---> algorithm parameter
	
	int						m_SmallWinSize;		/*!< A positive number specifying the lower bound of the support region.*/
	std::vector<float>		m_ParamNorm;		/*!< A vector specifying the shrinked Hampel norm parameter &sigma;<sub>1</sub> and &sigma;<sub>2</sub>*/
	std::vector<float>		m_Param;			/*!< A vector specifying additional program parameter.*/
	float					m_SSDThreshold;		/*!< Obsolete.*/
	int						m_NoL2Iteration;	/*!< A positive number specifying the number of iterations done with a L2 norm for the iterative refinement.*/
	bool					m_WinSizeHasChanged;/*!< Internal usage.*/
	
	public:
	float					m_BetaGradient;		/*!< Internal usage.*/
#ifdef USE_BOOSTSERIALIZATION
	private:
	friend class boost::serialization::access;
	template<class Archive>
	void serialize(Archive & ar, const unsigned int version)
	{
		ar & boost::serialization::make_nvp("CLocalFlowParameter", boost::serialization::base_object<CLocalFlowParameter>(*this));
		ar & boost::serialization::make_nvp("m_SmallWinSize", m_SmallWinSize);
		ar & boost::serialization::make_nvp("m_ParamNorm", m_ParamNorm);
		ar & boost::serialization::make_nvp("m_Param", m_Param);
		ar & boost::serialization::make_nvp("m_SSDThreshold", m_SSDThreshold);
		ar & boost::serialization::make_nvp("m_NoL2Iteration", m_NoL2Iteration);
		ar & boost::serialization::make_nvp("m_WinSizeHasChanged", m_WinSizeHasChanged);
	}
	public:
	virtual std::string uid() const 
	{
		std::string res = CLocalFlowParameter::uid()
			 +boost::lexical_cast<std::string>(m_SmallWinSize) 
			 +boost::lexical_cast<std::string>(m_SolverType) 
			 +boost::lexical_cast<std::string>(m_SSDThreshold) 
			 +boost::lexical_cast<std::string>(m_NoL2Iteration) 
			 +boost::lexical_cast<std::string>(m_WinSizeHasChanged);
		for(auto i = m_ParamNorm.begin(); i != m_ParamNorm.end(); ++i)
			res = res + boost::lexical_cast<std::string>(*i);
		for(auto i = m_Param.begin(); i != m_Param.end(); ++i)
			res = res + boost::lexical_cast<std::string>(*i);
		return res;
	}
#endif
};
/*! \class CCrossParameter
 *  \brief Cross based PLK/RLOF parameter base class
 *  The cross based parameter base class for the Pyramidal Lucas Kanade and the Robust Local Optical Flow provides basic interfaces to set up the algorithm parameter with adaptive support region, see <a href="index.html#Senst2014">Senst et al. 2014</a>. 
 *  The parameter class is associated to CROSS_PLK, CROSS_BEPLK, CROSS_RLOF, CROSS_BERLOF.
*/
class DECL_SPEC CCrossParameter 
{
public:
	/*! Enumeration specifying a colour space used for support region construction.
	*/
	enum ColorSpace { COLOR_RGB = 0,/*!< RGB colour space */
					  COLOR_CIEXYZ  /*!< not used */
					};
	CCrossParameter();
	/*! Set the image value threshold in order to construct the adaptive support sizes.
	 *\param val threshold [0 255] refer to &tau;.
	*/
	void segmentationThreshold(int val);
	/*! Interface to retrieve the support region construction threshold.
	 *\return threshold &tau;.
	*/
	int segmentationThreshold() const;
	/*! Set the lower bound of the support region size.
	 *\param val a positive value.
	*/
	void smallWinSize(const int & val);
	/*! Interface to retrieve the upper lower of the support region.
	 *\return Minimal support regions size.
	*/
	const int smallWinSize() const ;
	/*! Not used.
	 \return Not used.
	*/
	bool usePreComputedCross() const { return m_UsePreComputedCross;}
protected:
	int			m_SegmentationThreshold;	/*!< A positive number specifying the segmentation threshold &tau; as described in <a href="index.html#Senst2014">Senst et al. 2014</a>.*/
	bool		m_UseBothImage;				/*!< Not used */	
	bool		m_UsePreComputedCross;		/*!< Not used */
	ColorSpace	m_ColorSpace;				/*!< Only RGB colour space will be supported.*/
	int			m_SmallWinSize;				/*!< A positive number specifying the lower bound of the support region.*/
	#ifdef USE_BOOSTSERIALIZATION
	private:
	friend class boost::serialization::access;
	template<class Archive>
	void serialize(Archive & ar, const unsigned int version)
	{
		ar & boost::serialization::make_nvp("m_SegmentationThreshold", m_SegmentationThreshold);
		ar & boost::serialization::make_nvp("m_UseBothImage", m_UseBothImage);
		ar & boost::serialization::make_nvp("m_UsePreComputedCross", m_UsePreComputedCross);
		ar & boost::serialization::make_nvp("m_ColorSpace", m_ColorSpace);
		ar & boost::serialization::make_nvp("m_SmallWinSize", m_SmallWinSize);
	}
	public:
	virtual std::string uid() const 
	{
		return boost::lexical_cast<std::string>(m_SegmentationThreshold) 
			 +boost::lexical_cast<std::string>(m_UseBothImage) 
			 +boost::lexical_cast<std::string>(m_UsePreComputedCross) 
			 +boost::lexical_cast<std::string>(m_ColorSpace) 
			 +boost::lexical_cast<std::string>(m_SmallWinSize);
		
	}
#endif
};

/*! \class CPLKParameter
 *  \brief Pyramidal Lucas Kanade parameter class
 *  The Pyramidal Lucas Kanade parameter class. Please use a instance of this class to run the PLK version. 
 *  The parameter class is associated to PLK.
*/
class DECL_SPEC CPLKParameter : public CLocalFlowParameter
{
public:
	/*! Set the rectangle support region size.
	 *\param val A positive number specifying the support region.
	*/
	void winSize(int val) { m_WinSize =val;};
	/*! Interface to retrieve the support region size.
	 *\return support region size.
	*/
	int winSize() const{ return m_WinSize;};
#ifdef USE_BOOSTSERIALIZATION
	private:
	friend class boost::serialization::access;
	template<class Archive>
	void serialize(Archive & ar, const unsigned int version)
	{
		ar & boost::serialization::make_nvp("CLocalFlowParameter", boost::serialization::base_object<CLocalFlowParameter>(*this));
	}
#endif
};

/*! \class CCrossRobustFlowParameter
 *  \brief Cross based RLOF parameter class
 *  The cross based Robust Local Optical Flow parameter class contains interfaces to additional parameters, see <a href="index.html#Senst2014">Senst et al. 2014</a> . 
 *  The parameter class is associated to CROSS_RLOF, CROSS_BERLOF.
*/
class DECL_SPEC CCrossRobustFlowParameter: public CCrossParameter , public CRobustFlowParameter
{
public:
	CCrossRobustFlowParameter();
	/*! Set the lower bound of the adaptive support region size.
	 *\param val a positive number.
	*/
	void smallWinSize(const int & val);
	/*! Interface to retrieve the lower bound of the adaptive support region.
	 *\return Minimal support regions size.
	*/
	const int smallWinSize() const ;
#ifdef USE_BOOSTSERIALIZATION
	private:
	friend class boost::serialization::access;
	template<class Archive>
	void serialize(Archive & ar, const unsigned int version)
	{
		ar & boost::serialization::make_nvp("CRobustFlowParameter", boost::serialization::base_object<CRobustFlowParameter>(*this));
		ar & boost::serialization::make_nvp("CCrossParameter", boost::serialization::base_object<CCrossParameter>(*this));
	}
	public:
	virtual std::string uid() const 
	{
		return CCrossParameter::uid() + CRobustFlowParameter::uid();
	}
#endif
};
/*! \class CCrossPLKParameter
 *  \brief Cross based Pyramidal Lucas Kanade parameter class
 *  The cross based Pyramidal Lucas Kanade parameter class contains additional parameters, see <a href="index.html#Senst2012">Senst et al. 2012</a>. 
 *  The parameter class is associated to CROSS_RLOF, CROSS_BERLOF.
*/
class DECL_SPEC CCrossPLKParameter : public CCrossParameter, public CPLKParameter
{
public:
	CCrossPLKParameter();
#ifdef USE_BOOSTSERIALIZATION
	private:
	friend class boost::serialization::access;
	template<class Archive>
	void serialize(Archive & ar, const unsigned int version)
	{
		ar & boost::serialization::make_nvp("CPLKParameter", boost::serialization::base_object<CPLKParameter>(*this));
		ar & boost::serialization::make_nvp("CCrossParameter", boost::serialization::base_object<CCrossParameter>(*this));	
	}
	public:
	virtual std::string uid() const 
	{
		return CCrossParameter::uid()+ CLocalFlowParameter::uid();
	}
#endif
};

/*! \class CLocalFlowParameterFactory
 *  \brief RLOF parameter factory
 *  A factory to facilitate the serialization of the local optical flow parameter classes. 
*/
class CLocalFlowParameterFactory
{
public:
	/*! Helper function to assoziate a string to a parameter instance.
	 *\param parameter Pointer to an instance of CLocalFlowParameter derived class.
	*/
	static std::string name(const FeatureTrackerContext * parameter)
	{
		if( dynamic_cast<const CPLKParameter*>(parameter) )
			return "CPLKParameter";
		if( dynamic_cast<const CCrossPLKParameter*>(parameter) )
			return "CCrossPLKParameter";
		if( dynamic_cast<const CCrossRobustFlowParameter*>(parameter) )
			return "CCrossRobustFlowParameter";
		if( dynamic_cast<const CCrossParameter*>(parameter) )
			return "CCrossParameter";
		if( dynamic_cast<const CRobustFlowParameter*>(parameter) )
			return "CRobustFlowParameter";
		return "";
	}
	/*! Create a PLK/RLOF parameter class specified with a string.
	 *\param name a string that indicates the parameter class version,
	          - CPLKParameter : CPLKParameter
			  - CCrossPLKParameter : CCrossPLKParameter
			  - CCrossRobustFlowParameter : CCrossRobustFlowParameter
			  - CRobustFlowParameter : CRobustFlowParameter
	*/
	static CLocalFlowParameter * create(std::string name)
	{
		if( name == "CPLKParameter")
			return new CPLKParameter;
		if( name == "CCrossPLKParameter")
			return new CCrossPLKParameter;
		if( name == "CCrossRobustFlowParameter")
			return new CCrossRobustFlowParameter;
		if( name == "CRobustFlowParameter" )
			return new CRobustFlowParameter;
		return NULL;
	}	
	#ifdef USE_BOOSTSERIALIZATION
	static void save(std::string basePath, const FeatureTrackerContext * parameter)
	{
		using boost::serialization::make_nvp;
		std::string _name = name(parameter);
		std::ofstream ofs((basePath.append(_name).append(".xml")).c_str());
		if( ofs.is_open() == false)
			throw(std::exception("[CLocalFlowParameterFactory::save] Could not open setting file"));

		boost::archive::xml_oarchive oa(ofs);
		if( dynamic_cast<const CPLKParameter*>(parameter) )
			oa << make_nvp(_name.c_str(), *dynamic_cast<const CPLKParameter*>(parameter));
		else if( dynamic_cast<const CCrossPLKParameter*>(parameter) )
			oa << make_nvp(_name.c_str(), *dynamic_cast<const CCrossPLKParameter*>(parameter));
		else if( dynamic_cast<const CCrossRobustFlowParameter*>(parameter) )
			oa << make_nvp(_name.c_str(), *dynamic_cast<const CCrossRobustFlowParameter*>(parameter));
		else if( dynamic_cast<const CCrossParameter*>(parameter) )
			oa << make_nvp(_name.c_str(), *dynamic_cast<const CCrossParameter*>(parameter));
		else if( dynamic_cast<const CRobustFlowParameter*>(parameter) )
			oa << make_nvp(_name.c_str(), *dynamic_cast<const CRobustFlowParameter*>(parameter));
		else
			throw(std::exception("[CLocalFlowParameterFactory::serialize] no cast found."));
	}
	static void load(std::string basePath, FeatureTrackerContext * parameter)
	{
		using boost::serialization::make_nvp;
		std::string _name = name(parameter);
		std::ifstream ifs((basePath.append(_name).append(".xml")).c_str());
		if( ifs.is_open() == false)
		{
			save(basePath, parameter);
			std::cerr << "[CLocalFlowParameterFactory::serialize] use and save default: " << basePath.append(_name).append(".xml") << std::endl;
			return;
		}

		boost::archive::xml_iarchive ia(ifs);
		if( dynamic_cast<CPLKParameter*>(parameter) )
			ia >> make_nvp(_name.c_str(), *dynamic_cast<CPLKParameter*>(parameter));
		else if( dynamic_cast<CCrossPLKParameter*>(parameter) )
			ia >> make_nvp(_name.c_str(), *dynamic_cast<CCrossPLKParameter*>(parameter));
		else if( dynamic_cast<CCrossRobustFlowParameter*>(parameter) )
			ia >> make_nvp(_name.c_str(), *dynamic_cast<CCrossRobustFlowParameter*>(parameter));
		else if( dynamic_cast<CCrossParameter*>(parameter) )
			ia >> make_nvp(_name.c_str(), *dynamic_cast<CCrossParameter*>(parameter));
		else if( dynamic_cast<CRobustFlowParameter*>(parameter) )
			ia >> make_nvp(_name.c_str(), *dynamic_cast<CRobustFlowParameter*>(parameter));
		else
			throw(std::exception("[CLocalFlowParameterFactory::serialize] no cast found."));
	}
	#endif
};

#endif