/******************************************************************************
 * COPYRIGHT:                                                                 * 
 *                                                                            *
 * This file is the property of the author and Communication Systems Group,   *
 * Technische Universitt Berlin. All rights reserved.                        *
 *                                                                            *
 * It may not be publicly disclosed, distributed, used, copied or modified    *
 * without prior written authorization by a representative of                 *
 * Communication Systems Group, Technische Universitt Berlin or the author.  *
 * Any modified version of this document needs to contain this header.        *
 ******************************************************************************/

/******************************************************************************
 * THERMS IF USAGE:                                                           *
 * PERSONAL, NON-COMMERCIAL or ACADEMIC USAGE:                                *
 *   You are free to use this software for whatever you like. If you use this *
 *   algorithm for a scientific publication, please cite the one of the       *
 *   following paper:                                                         *
 *																			  *
 * @INPROCEEDINGS{ICIPSenst2014,											  *
 * AUTHOR = {Tobias Senst and Thilo Borgmann and Ivo Keller and Thomas		  *
 * Sikora},																	  *
 * TITLE = {Cross based Robust Local Optical Flow},							  *
 * BOOKTITLE = {21th IEEE International Conference on Image Processing},	  *
 * YEAR = {2014},															  *
 * MONTH = okt,																  *
 * PAGES = {1967--1971},													  *
 * ADDRESS = {Paris, France},												  *
 * }																		  *
 *																			  *
 * @INPROCEEDINGS{ICIPSenst2013,											  *
 * AUTHOR = {Tobias Senst and Jonas Geistert and Ivo Keller and Thomas		  *
 * Sikora},																	  *
 * TITLE = {Robust Local Optical Flow Estimation using Bilinear Equations for *
 * Sparse Motion Estimation},												  *
 * BOOKTITLE = {20th IEEE International Conference on Image Processing},	  *
 * YEAR = {2013},															  *
 * MONTH = sep,																  *
 * PAGES = {2499--2503},													  *
 * ADDRESS = {Melbourne, Australia},										  *
 * DOI = {10.1109/ICIP.2013.6738515},										  *
 * }																		  *
 *                                                                            *
 * @ARTICLE{TCSVTSenst2012,                                                   *
 * AUTHOR = {Tobias Senst and Volker Eiselein and Thomas Sikora},             *
 * TITLE = {Robust Local Optical Flow for Feature Tracking},                  *
 * JOURNAL = {IEEE Transactions on Circuits and Systems for Video Technology},*
 * YEAR = {2012},                                                             *
 * MONTH = sep,                                                               *
 * PAGES = {1377--1387},                                                      *
 * VOLUME = {22},                                                             *
 * NUMBER = {9},                                                              *
 * DOI = {10.1109/TCSVT.2012.2202070}                                         *
 * }                                                                          *
 *                                                                            *
 * @INPROCEEDINGS{WACVSenst2011,                                              *
 * AUTHOR = {Tobias Senst and Volker Eiselein and Rubn Heras Evangelio and   *
 * Thomas Sikora},                                                            *
 * TITLE = {Robust Modified L2 Local Optical Flow Estimation and Feature      *
 * Tracking},                                                                 *
 * BOOKTITLE = {IEEE Workshop on Motion and Video Computing},                 *
 * YEAR = {2011},                                                             *
 * MONTH = jan,                                                               *
 * EDITOR = {Eric Mortensen},                                                 *
 * PAGES = {685--690},                                                        *
 * ADDRESS = {Kona, USA},                                                     *
 * DOI = {10.1109/WACV.2011.5711571},                                         *
 * }                                                                          *
 *                                                                            *
 * COMMERCIAL USAGE:                                                          *
 *   It is not allowed to use any content of this package for any commercial  *
 *   use or any advertisement for upcoming commercial products. If you want to*
 *   use any content for such a purpose please contact:                       *
 *   Prof. Dr.-Ing. Thomas Sikora <sikora@nue.tu-berlin.de>.                  *
 ******************************************************************************/

/******************************************************************************
 * WARRANTIES:                                                                *
 *                                                                            *
 * Software provided by Technische Universitt Berlin with this document is   *
 * provided "AS IS" and any express of implied warranties including, but      *
 * not limited to, the implied warranties of merchantability and fitness      *
 * for a particular purpose are disclaimed.                                   *
 * In no event shall the author or contributors be liable for any direct,     *
 * indirect, incidental, special, exemplary, or consequential damages         *
 * (including, but not limited to, procurement of substitute goods or         *
 * services, loss of use, data, or profits or business interruption) caused in*
 * any way out of the use of this software, even if advised of the possibility*
 * of such damage.                                                            *
 ******************************************************************************/
/** 
 * \file RLOFTools.hpp
 * \brief Motion estimation and illustration.
   
   Contains a set of useful function regarding motion estimation and illustration.
 * \version 1.2
 * \date 04.05.2012
 * \author Tobias Senst
 */

#pragma once
#ifndef _RLOFTOOLS_HPP_
#define _RLOFTOOLS_HPP_
#include <OpticalFlow/RLOFFactory.h>
#include <OpticalFlow/RLOFParameter.h>
#include <iostream>
#include <opencv\highgui.h>
/*! Compute a dense motion field with the prefered optical flow method
 *\param img1 Image at time t.
 *\param img2 Image at time t+1.
 *\param tracker Pointer to the RLOF instance.
 *\param param RLOF parameter.
 *\param U Resulting u matrice containing dx motions of type CV_32FC1.
 *\param V Resulting v matrice containing dy motions of type CV_32FC1.
*/
static void denseMotionField(const cv::Mat & img1, 
							 const cv::Mat & img2, 
							 IFeatureTracker * tracker, 
							 FeatureTrackerContext & param, 
							 cv::Mat & U,
							 cv::Mat & V)
{
	std::vector<CRPoint> ptList1(img1.size().area());
	std::vector<CRPoint> ptList2(img1.size().area());
	// initialise pointlist
	int n = 0;
	for(int r = 0 ; r < img1.rows; r++)
	{
		for(int c = 0 ; c < img1.cols; c++, n++)
		{
			ptList1[n] = CRPoint(static_cast<float>(c),static_cast<float>(r));
		}
	}
	
	tracker->run(img1.ptr<uchar>(),
			  img1.step1(),
			  img2.ptr<uchar>(),
			  img2.step1(),
			  img1.rows,
			  img1.cols,
			  ptList1,
			  ptList2,
			  &param);

	// initialise the motion field matrices
	if( U.size() != img1.size() || U.type() != CV_32FC1)
	{
		U.release();
		U.create(img1.size(), CV_32FC1);
		U.setTo(0);
	}
	if( V.size() != img1.size() || V.type() != CV_32FC1)
	{
		V.release();
		V.create(img1.size(), CV_32FC1);
		V.setTo(0);
	}
	// 
 	for(unsigned int n = 0; n < ptList1.size(); n++)
	{
		cv::Point pt(static_cast<int>(ptList1[n].x), static_cast<int>(ptList1[n].y));
		U.at<float>(pt) = ptList2[n].x - ptList1[n].x;
		V.at<float>(pt) = ptList2[n].y - ptList1[n].y;
	}
}



//! Convert motion field to RGB colormap
/*! Function converts a motion vector field specified by dx and dy to a 8UC3 RGB colormap using the HSV2RGB transformation. \anchor FlowToRGB
 *  While Saturation(S) and Value(V) channels are used to code the length of a motion vector, Hue(H) is used to display the direction (angle)
 *  Therefore lenght of a motion vector is scaled with max_size
 *	\param dx 32FC1 motion vectorfield dx component
 *	\param dy 32FC1 motion vectorfield dx component
 *	\param rgbFlow 8UC3 output matrice (colormap)
 *	\param max_size Maximal value to rescale the S or V channel.
 *	\param use_value If set V channels is used to code motion vector length else S channels is used
 *	\param sat If set it has to be 8UC1 and is used the weight the sat channel (by default it should not be used)
*/
void FlowToRGB(const cv::Mat & dx,
				const cv::Mat & dy, 
				cv::Mat & rgbFlow,
				const float & max_size = -1,
				bool use_value = false,
				cv::Mat & sat = cv::Mat())
{
	if(dx.empty() ||dy.empty()) return;

	if( dx.depth() != CV_32F && dy.depth() != CV_32F)
		throw(std::exception("FlowToRGB: error dx/dy wrong data type"));
	if( dx.size() != dy.size())
		throw(std::exception("FlowToRGB: error dx/dy must have equal size"));
	if ( !sat.empty() )
		if( sat.type() != CV_8UC1)
			throw(std::exception("FlowToRGB: error sat must have type CV_8UC1"));


	const float grad2deg = (float)(90/3.141);
	//const float norm_val = (float)(255.f / max_val);
	double satMaxVal, minVal;
	if(!sat.empty())
	{
		cv::minMaxLoc(sat, &minVal, &satMaxVal);
		satMaxVal = 255.0/satMaxVal;
	}

	cv::Mat pol(dx.size(), CV_32FC2);

	float mean_val = 0, min_val = 1000, max_val = 0;
	float _dx, _dy, angle, value;

	for(int r = 0; r < dx.rows; r++)
	{
		for(int c = 0; c < dy.cols; c++)
		{
			_dx = dx.at<float>(r,c);
			_dy = dy.at<float>(r,c);
			value =sqrt(_dx * _dx + _dy * _dy);
			if ( value == 0)
				angle = 0;
			else if( _dy < 0)
				angle = 180.f -  acos(_dx/ value)* grad2deg;
			else
				angle = acos(_dx/ value)* grad2deg;
			
			mean_val +=value;
			max_val = MAX(max_val, value);
			min_val = MIN(min_val, value);
			pol.at<cv::Point2f>(r,c) = cv::Point2f(angle,value);
		}
	}
	mean_val /= dy.size().area();
	float scale = max_val - min_val;
	float shift = -min_val;//-mean_val + scale;
	scale = 255.f/scale;
	if( max_size > 0)
	{
		scale = 255.f/max_size;
		shift = 0;
	}
	
	//calculate the angle, motion value 
	cv::Mat hsv(dx.rows, dx.cols, CV_8UC3);
	uchar * ptrHSV = hsv.ptr<uchar>();
	int idx_val  = (use_value) ? 2:1;
	int idx_sat  = (use_value) ? 1:2;

	
	for(int r = 0; r < dx.rows; r++, ptrHSV += hsv.step1())
	{
		uchar * _ptrHSV = ptrHSV;
		for(int c = 0; c < dy.cols; c++, _ptrHSV+=3)
		{
			cv::Point2f vpol = pol.at<cv::Point2f>(r,c);

			_ptrHSV[0] = cv::saturate_cast<uchar>(vpol.x);
			_ptrHSV[idx_val] = cv::saturate_cast<uchar>( (vpol.y + shift) * scale);  
			if( sat.empty())
				_ptrHSV[idx_sat] = 255;
			else
				_ptrHSV[idx_sat] = 255-  cv::saturate_cast<uchar>(sat.at<uchar>(r,c) * satMaxVal); 

		}
	}
	
	//std::vector<cv::Mat> vec;
	//cv::split(hsv, vec);
	//cv::equalizeHist(vec[idx_val],vec[idx_val]);
	//cv::merge(vec,hsv);
	cv::Mat rgbFlow32F;
	cv::cvtColor(hsv, rgbFlow32F, CV_HSV2BGR);
	rgbFlow32F.convertTo(rgbFlow, CV_8UC3);
}
/*! Test function. Test the consitens of the accuracy of the library.
*/
void test1()
{
	std::string filename1("../Doc/ErnstReuter1.png");
	std::string filename2("../Doc/ErnstReuter2.png");
	cv::Mat img1;
	cv::Mat img2;
	cv::Mat U,V;
	img1 = cv::imread(filename1);
	img2 = cv::imread(filename2);

	CPLKParameter				PLKParameter;			// PLK OpenCV implementation refer to  CPU_PLK / GPU_PLK
	CRobustFlowParameter		RLOFParameter;			// RLOF see <a href="index.html#Senst2012">Senst et al. 2012</a> refer to CPU_RLOF / GPU_RLOF

	CPLKParameter				BEPLKParameter;		// BEPLK  see <a href="index.html#Senst2013">Senst et al. 2013</a> refer to CPU_BEPLK
	BEPLKParameter.solverType(CLocalFlowParameter::ITER_BE);
	CRobustFlowParameter		BERLOFParameter;		// BERLOF see <a href="index.html#Senst2013">Senst et al. 2013</a> refer to CPU_BERLOF / GPU_BERLOF
	BERLOFParameter.solverType(CLocalFlowParameter::ITER_BE);

	CCrossPLKParameter			CB_BEPLKParameter;		// CB_BEPLK  see /cite{ICIPSenst2014} refer to CPU_CROSS_BEPLK
	CB_BEPLKParameter.solverType(CLocalFlowParameter::ITER_BE);
	CCrossRobustFlowParameter	CB_BERLOFParameter;		// CB_BERLOF see /cite{ICIPSenst2014} refer to CPU_CROSS_BERLOF
	CB_BERLOFParameter.solverType(CLocalFlowParameter::ITER_BE);
	CCrossPLKParameter			CB_PLKParameter;		// CB_PLK  see /cite{ICIPSenst2014} refer to CPU_CROSS_PLK
	CCrossRobustFlowParameter	CB_RLOFParameter;		// CB_RLOF see /cite{ICIPSenst2014} refer to CPU_CROSS_RLOF

	CLocalFlowParameter * parameters[8] = {&PLKParameter,&RLOFParameter, &BEPLKParameter, &BERLOFParameter, &CB_BEPLKParameter, &CB_BERLOFParameter, &CB_PLKParameter, &CB_RLOFParameter};
	std::string methodName[8]			= {"PLK", "RLOF", "BEPLK", "BERLOF", "CB_BEPLK", "CB_BERLOF", "CB_PLK", "CB_RLOF"};
	std::cout << "--- RLOFLib Accuracy Test ---" << std::endl;
	FILE * file;

#ifdef _WIN32 || _WIN64
#if _WIN64
		fopen_s(&file, "../Doc/RLOFTest64.dat", "r");
#else
		fopen_s(&file, "../Doc/RLOFTest32.dat", "r");
#endif
#endif

	
	for(int n = 0; n < 8; n++)
	{
		IFeatureTracker * tracker = RLOFFactory::createFTInstance(parameters[n]);
		denseMotionField(img1, img2, tracker, *parameters[n], U, V);
		delete tracker;
		float R = 0;
		for( int r = 0; r < U.rows; r++)
			for( int c = 0; c < U.cols; c++)
			{
				float u, v, b;
				fscanf(file, "%f %f %f ",&u, &v, &b);
				float mu = U.at<float>(r,c);
				float mv = V.at<float>(r,c);
				if( sqrt((u - mu) * (u - mu) + (v - mv) * (v - mv)) > 0.01 && b != 0)
					R++;

			}
		R /= static_cast<float>(U.size().area());
		if( R < 0.001 )
			std::cout << methodName[n] << "\t\tTEST OK!" << std::endl;
		else
			std::cout << methodName[n] << "\t\tATTENTION R = "<< R << std::endl;
	}
	fclose(file);
}
/*! Test function. Check for all intances available.
*/
void test0()
{
	CPLKParameter				PLKParameter;			// PLK OpenCV implementation refer to  CPU_PLK / GPU_PLK
	CRobustFlowParameter		RLOFParameter;			// RLOF see <a href="index.html#Senst2012">Senst et al. 2012</a> refer to CPU_RLOF / GPU_RLOF

	CPLKParameter				BEPLKParameter;		// BEPLK  see <a href="index.html#Senst2013">Senst et al. 2013</a> refer to CPU_BEPLK
	BEPLKParameter.solverType(CLocalFlowParameter::ITER_BE);
	CRobustFlowParameter		BERLOFParameter;		// BERLOF see <a href="index.html#Senst2013">Senst et al. 2013</a> refer to CPU_BERLOF / GPU_BERLOF
	BERLOFParameter.solverType(CLocalFlowParameter::ITER_BE);

	CCrossPLKParameter			CB_BEPLKParameter;		// CB_BEPLK  see /cite{ICIPSenst2014} refer to CPU_CROSS_BEPLK
	CB_BEPLKParameter.solverType(CLocalFlowParameter::ITER_BE);
	CCrossRobustFlowParameter	CB_BERLOFParameter;		// CB_BERLOF see /cite{ICIPSenst2014} refer to CPU_CROSS_BERLOF
	CB_BERLOFParameter.solverType(CLocalFlowParameter::ITER_BE);
	CCrossPLKParameter			CB_PLKParameter;		// CB_PLK  see /cite{ICIPSenst2014} refer to CPU_CROSS_PLK
	CCrossRobustFlowParameter	CB_RLOFParameter;		// CB_RLOF see /cite{ICIPSenst2014} refer to CPU_CROSS_RLOF

	CLocalFlowParameter * parameters[8] = {&PLKParameter,&RLOFParameter, &BEPLKParameter, &BERLOFParameter, &CB_BEPLKParameter, &CB_BERLOFParameter, &CB_PLKParameter, &CB_RLOFParameter};
	std::string methodName[8]			= {"PLK", "RLOF", "BEPLK", "BERLOF", "CB_BEPLK", "CB_BERLOF", "CB_PLK", "CB_RLOF"};

	std::cout << "--- RLOFLib Initialisation Test ---" << std::endl;
	for(int n = 0; n < 8; n++)
	{
		try
		{
			IFeatureTracker * tracker = RLOFFactory::createFTInstance(parameters[n]);
			delete tracker;
			std::cout << methodName[n] << "\t\tTEST OK!" << std::endl;
		}
		catch(std::exception & e)
		{
			std::cout << methodName[n] << "\t\tTEST FAILED!" << std::endl << e.what() <<std::endl;
		}
	}
}

#endif