/******************************************************************************
 * COPYRIGHT:                                                                 * 
 *                                                                            *
 * This file is the property of the author and Communication Systems Group,   *
 * Technische Universitt Berlin. All rights reserved.                        *
 *                                                                            *
 * It may not be publicly disclosed, distributed, used, copied or modified    *
 * without prior written authorization by a representative of                 *
 * Communication Systems Group, Technische Universitt Berlin or the author.  *
 * Any modified version of this document needs to contain this header.        *
 ******************************************************************************/

/******************************************************************************
 * THERMS IF USAGE:                                                           *
 * PERSONAL, NON-COMMERCIAL or ACADEMIC USAGE:                                *
 *   You are free to use this software for whatever you like. If you use this *
 *   algorithm for a scientific publication, please cite the one of the       *
 *   following paper:                                                         *
 *																			  *
 * @INPROCEEDINGS{ICIPSenst2014,											  *
 * AUTHOR = {Tobias Senst and Thilo Borgmann and Ivo Keller and Thomas		  *
 * Sikora},																	  *
 * TITLE = {Cross based Robust Local Optical Flow},							  *
 * BOOKTITLE = {21th IEEE International Conference on Image Processing},	  *
 * YEAR = {2014},															  *
 * MONTH = okt,																  *
 * PAGES = {1967--1971},													  *
 * ADDRESS = {Paris, France},												  *
 * }																		  *
 *																			  *
 * @INPROCEEDINGS{ICIPSenst2013,											  *
 * AUTHOR = {Tobias Senst and Jonas Geistert and Ivo Keller and Thomas		  *
 * Sikora},																	  *
 * TITLE = {Robust Local Optical Flow Estimation using Bilinear Equations for *
 * Sparse Motion Estimation},												  *
 * BOOKTITLE = {20th IEEE International Conference on Image Processing},	  *
 * YEAR = {2013},															  *
 * MONTH = sep,																  *
 * PAGES = {2499--2503},													  *
 * ADDRESS = {Melbourne, Australia},										  *
 * DOI = {10.1109/ICIP.2013.6738515},										  *
 * }																		  *
 *                                                                            *
 * @ARTICLE{TCSVTSenst2012,                                                   *
 * AUTHOR = {Tobias Senst and Volker Eiselein and Thomas Sikora},             *
 * TITLE = {Robust Local Optical Flow for Feature Tracking},                  *
 * JOURNAL = {IEEE Transactions on Circuits and Systems for Video Technology},*
 * YEAR = {2012},                                                             *
 * MONTH = sep,                                                               *
 * PAGES = {1377--1387},                                                      *
 * VOLUME = {22},                                                             *
 * NUMBER = {9},                                                              *
 * DOI = {10.1109/TCSVT.2012.2202070}                                         *
 * }                                                                          *
 *                                                                            *
 * @INPROCEEDINGS{WACVSenst2011,                                              *
 * AUTHOR = {Tobias Senst and Volker Eiselein and Rubn Heras Evangelio and   *
 * Thomas Sikora},                                                            *
 * TITLE = {Robust Modified L2 Local Optical Flow Estimation and Feature      *
 * Tracking},                                                                 *
 * BOOKTITLE = {IEEE Workshop on Motion and Video Computing},                 *
 * YEAR = {2011},                                                             *
 * MONTH = jan,                                                               *
 * EDITOR = {Eric Mortensen},                                                 *
 * PAGES = {685--690},                                                        *
 * ADDRESS = {Kona, USA},                                                     *
 * DOI = {10.1109/WACV.2011.5711571},                                         *
 * }                                                                          *
 *                                                                            *
 * COMMERCIAL USAGE:                                                          *
 *   It is not allowed to use any content of this package for any commercial  *
 *   use or any advertisement for upcoming commercial products. If you want to*
 *   use any content for such a purpose please contact:                       *
 *   Prof. Dr.-Ing. Thomas Sikora <sikora@nue.tu-berlin.de>.                  *
 ******************************************************************************/

/******************************************************************************
 * WARRANTIES:                                                                *
 *                                                                            *
 * Software provided by Technische Universitt Berlin with this document is   *
 * provided "AS IS" and any express of implied warranties including, but      *
 * not limited to, the implied warranties of merchantability and fitness      *
 * for a particular purpose are disclaimed.                                   *
 * In no event shall the author or contributors be liable for any direct,     *
 * indirect, incidental, special, exemplary, or consequential damages         *
 * (including, but not limited to, procurement of substitute goods or         *
 * services, loss of use, data, or profits or business interruption) caused in*
 * any way out of the use of this software, even if advised of the possibility*
 * of such damage.                                                            *
 ******************************************************************************/
/** 
 * \file RLOFSample.cpp
 * \brief RLOF sample implementation.
 
 Provide a sample implementation, how to use the RLOF libary.
 * \version 1.2
 * \date 04.05.2012
 * \author Tobias Senst
 */


#include <OpticalFlow/RLOFTools.hpp>
#include <iostream>

/*! Enum to select feature selection method for the sample programm
*/
enum TrackerType { DENSE = 0 /*!< Compute for each pixel of the input image motion vectors.*/
				 , GFT /*!< Detect features with the Good Features To Track method and compute motion vectors for these features.*/
				 };

//! Demo programm  
/*! Sample to demonstrate the usage of the RLOF libary. The programm designed to compute the motion vectors of:
 - a dense motion field
 - feature points selected by a Good Feature to Track method 
 with the GPU or CPU version of the PLK/RLOF methods providen by the libary. 
*/
void demo()
{
	const TrackerType trackerType = DENSE; // or DENSE

	const RLOFType rlofType	= CPU_PLK; // or CPU_CROSS_BEPLK ...

	// create parameter instances for each type of PLK/RLOF method (exampleS)
	CPLKParameter				PLKParameter;			// PLK OpenCV implementation refer to  CPU_PLK / GPU_PLK
	CRobustFlowParameter		RLOFParameter;			// RLOF see <a href="index.html#Senst2012">Senst et al. 2012</a> refer to CPU_RLOF / GPU_RLOF

	CPLKParameter				BEPLKLParameter;		// BEPLK  see <a href="index.html#Senst2013">Senst et al. 2013</a> refer to CPU_BEPLK
	BEPLKLParameter.solverType(CLocalFlowParameter::ITER_BE);
	CRobustFlowParameter		BERLOFParameter;		// BERLOF see <a href="index.html#Senst2013">Senst et al. 2013</a> refer to CPU_BERLOF / GPU_BERLOF
	BERLOFParameter.solverType(CLocalFlowParameter::ITER_BE);

	CCrossPLKParameter			CB_PLKParameter;		// CB_BEPLK  see <a href="index.html#Senst2014">Senst et al. 2014</a> refer to CPU_CROSS_BEPLK
	CB_PLKParameter.solverType(CLocalFlowParameter::ITER_BE);
	CCrossRobustFlowParameter	CB_RLOFParameter;		// CB_BERLOF see <a href="index.html#Senst2014">Senst et al. 2014</a> refer to CPU_CROSS_BERLOF
	CB_RLOFParameter.solverType(CLocalFlowParameter::ITER_BE);
		
	IFeatureTracker     * tracker = NULL;
	// select the method HERE
	CLocalFlowParameter * trackerParameter = &CB_PLKParameter;
	try
	{
		std::string filename1("../Doc/ErnstReuter1.png");
		std::string filename2("../Doc/ErnstReuter2.png");
		cv::Mat img1;
		cv::Mat img2;
		cv::Mat rgbOutput;
		img1 = cv::imread(filename1);
		img2 = cv::imread(filename2);

		// initialise PLK/RLOF method old version
		//IFeatureTracker * rlof = RLOFFactory::createFTInstance(rlofType);
		// initialise PLK/RLOF method new version. NOTE Only a view methods support the GPU. 
		tracker = RLOFFactory::createFTInstance(trackerParameter);

		/* the parameter could be modified here e.g
		CB_PLKParameter.level(1);
		*/

		if( trackerType == DENSE)
		{
			// run rlof for the whole image
			cv::Mat U;
			cv::Mat V;
			double time = static_cast<double>(cv::getTickCount());	
			denseMotionField(img1, img2, tracker, *trackerParameter, U, V);
			std::cout << "Dense (ms): " << (static_cast<double>(cv::getTickCount()) - time) / (cvGetTickFrequency() * 1000.) << std::endl;
		
			// colormap conversion
			FlowToRGB(U, V, rgbOutput, 5);
		}	
		else if (trackerType == GFT)
		{
			// run KLT like Good Features to Track + RLOF
			std::vector<cv::Point2f> ptList;
			std::vector<CRPoint> ptList1;
			std::vector<CRPoint> ptList2;
			// detect Good Features to Track 
			double time = static_cast<double>(cv::getTickCount());	
			cv::goodFeaturesToTrack(img1, ptList, 15000, 0.01, 3);
			cvTocr(ptList1, ptList);
			// track them
			
			tracker->run(img1.ptr<uchar>(),
					  img1.step1(),
					  img2.ptr<uchar>(),
					  img2.step1(),
					  img1.rows,
					  img1.cols,
					  ptList1,
					  ptList2,
					  &CB_PLKParameter);
			// draw the result
			std::cout << "GFT (ms): " << (static_cast<double>(cv::getTickCount()) - time) / (cvGetTickFrequency() * 1000.) << std::endl;
			cv::cvtColor(img1,rgbOutput, CV_GRAY2RGB);
			for(unsigned int n = 0 ; n < ptList1.size(); n++)
			{
				cv::Point pt1(static_cast<int>(ptList1[n].x), static_cast<int>(ptList1[n].y));
				cv::Point pt2(static_cast<int>(ptList2[n].x), static_cast<int>(ptList2[n].y));
				cv::circle(rgbOutput, pt1, 2, cv::Scalar(255,0,0), -1);
				cv::line(rgbOutput, pt1, pt2, cv::Scalar(0, 255, 0), 1);
			}
			
		}
		delete tracker;
		// display results
		cv::namedWindow("Motion Field",0);
		cv::imshow("Motion Field", rgbOutput);
		cv::waitKey(-1);
		
	}
	catch(std::exception & e)
	{
		std::cout << e.what()<<std::endl;
		system("pause");
		delete tracker;
	}
}

/*!
 *\param argc Ignored
 *\param argv Ignored
 *\return standart programm return vaule
*/
int main(int argc, char** argv) 
{
	// Perform initialization test
	test0();
	// Perform accuracy test
	test1();
	// Example 
	demo();
	return 1;
}

